package cn.daoway.sp;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;

/**
 * Servlet implementation class OrderServlet
 */
public class OrderServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;
       
    /**
     * @see HttpServlet#HttpServlet()
     */
    public OrderServlet() {
        super();
    }
    
    private OrderInfo parseNewOrderInfo(HttpServletRequest request){
    	OrderInfo orderInfo = new OrderInfo();
    	orderInfo.setAddress(request.getParameter("address"));
    	orderInfo.setAppointTime(request.getParameter("appointTime"));
    	orderInfo.setContactPerson(request.getParameter("contactPerson"));
    	orderInfo.setPhone(request.getParameter("phone"));
    	orderInfo.setNote(request.getParameter("note"));
    	String strItems = request.getParameter("items");
    	if(strItems!=null&&strItems.length()>0){
    		Gson gson = new Gson();
    		List<OrderItemInfo> items = gson.fromJson(strItems, 
    				new TypeToken<List<OrderItemInfo>>(){}.getType());
    		orderInfo.setItems(items);
    	}
    	return orderInfo;
    }
    
    public static int buildRandom(int length) {
		int num = 1;
		double random = Math.random();
		if (random < 0.1) {
			random = random + 0.1;
		}
		for (int i = 0; i < length; i++) {
			num = num * 10;
		}
		return (int) ((random * num));
	}
    
	public static String genUniqueOrderId(){
		//，
		SimpleDateFormat sdf =new SimpleDateFormat("yyyyMMddHHmmssSSS");
		return sdf.format(new Date())+buildRandom(7);
	}
    
    private void doCreateOrder(HttpServletRequest request, 
    		HttpServletResponse response) throws IOException{
    	Map<String,String> resMap = new HashMap<String,String>();
    	try{
    		//获取订单信息
        	OrderInfo orderInfo = parseNewOrderInfo(request);
        	// For debug
        	Gson gson = new Gson();
        	System.out.println("create order: " + gson.toJson(orderInfo));
        	
        	//创建实际订单,返回订单ID,由合作伙伴实现
        	//......
        	String orderId = genUniqueOrderId();
        	
        	
        	resMap.put("status", "ok");
        	resMap.put("orderId", orderId);	
    	}catch(Exception e){
    		resMap.put("status", "error");
        	resMap.put("msg", e.getMessage());	
    	}
    	Gson gson = new Gson();
    	response.setContentType("application/json");
    	response.getWriter().print(gson.toJson(resMap));
    }
    
    private void doCancelOrder(HttpServletRequest request, 
    		HttpServletResponse response) throws IOException{
    	String orderId = request.getParameter("orderId");
    	System.out.println("cancel order " +orderId);
    	//取消订单，具体逻辑由合作伙伴实现
    	//....
    	
    	Map<String,String> resMap = new HashMap<String,String>();
    	resMap.put("status", "ok");
    	Gson gson = new Gson();
    	response.setContentType("application/json");
    	response.getWriter().print(gson.toJson(resMap));
    }
    
	/**
	 * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		request.setCharacterEncoding("UTF-8");
		response.setCharacterEncoding("UTF-8");
		
		String path = request.getPathInfo();
		if(!DaowayUtils.verifySign(request,DaowayConfig.appkey,
				DaowayConfig.appsecret)){
			response.getWriter().println("{\"status\":\"error\",\"msg\":\"sign verify failed\"}");
			return;
		}
		if(path.equals("/create")){
			doCreateOrder(request, response);
		}else if(path.equals("/cancel")){
			doCancelOrder(request,response);
		}
	}
	

}
