package cn.daoway.sp;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import com.google.gson.Gson;

public class OrderStatusNotifier {

	private static final String DAOWAY_URL = "http://api.daoway.cn/daoway/rest/order_notify";

	// 除了appkey、oncestr、sign外，还有：
	// 订单id (orderId)
	// 订单状态 (status, 值可以为ongoing,canceled和completed)
	// 原因(note)
	public boolean notifyDaoWay(String orderId, OrderStatus newStatus,
			String note) {
		try {
			URL localURL = new URL(DAOWAY_URL);
			URLConnection connection = localURL.openConnection();
			HttpURLConnection httpURLConnection = (HttpURLConnection) connection;
			SortedMap<String, String> params = new TreeMap<String, String>();
			params.put("orderId", orderId);
			params.put("status", newStatus.toString());
			if (note != null && !note.isEmpty()) {
				params.put("note", note);
			}
			String parameterData = DaowayUtils.sign(params,
					DaowayConfig.appkey, DaowayConfig.appsecret);

			System.out.println("post parameters: " + parameterData);
			httpURLConnection.setDoOutput(true);

			httpURLConnection.setRequestMethod("POST");
			httpURLConnection.setRequestProperty("Accept-Charset", "utf-8");
			httpURLConnection.setRequestProperty("Content-Type",
					"application/x-www-form-urlencoded");
			httpURLConnection.setRequestProperty("Content-Length",
					String.valueOf(parameterData.length()));

			OutputStream outputStream = null;
			OutputStreamWriter outputStreamWriter = null;
			InputStream inputStream = null;
			InputStreamReader inputStreamReader = null;
			BufferedReader reader = null;
			try {
				outputStream = httpURLConnection.getOutputStream();
				outputStreamWriter = new OutputStreamWriter(outputStream);

				outputStreamWriter.write(parameterData.toString());
				outputStreamWriter.flush();

				if (httpURLConnection.getResponseCode() >= 300) {
					throw new Exception(
							"HTTP Request is not success, Response code is "
									+ httpURLConnection.getResponseCode());
				}
				inputStream = httpURLConnection.getInputStream();
				inputStreamReader = new InputStreamReader(inputStream);
				reader = new BufferedReader(inputStreamReader);
				String tempLine = null;
				StringBuilder resultBuffer = new StringBuilder();
				while ((tempLine = reader.readLine()) != null) {
					resultBuffer.append(tempLine);
				}
				System.out.println("response from daoway: "
						+ resultBuffer.toString());
				Gson json = new Gson();
				@SuppressWarnings("unchecked")
				Map<String, String> result = json.fromJson(
						resultBuffer.toString(), Map.class);
				if (result != null && result.get("status") != null
						&& result.get("status").equals("ok")) {
					return true;
				}
			} finally {
				if (outputStreamWriter != null) {
					outputStreamWriter.close();
				}

				if (outputStream != null) {
					outputStream.close();
				}
				if (reader != null) {
					reader.close();
				}

				if (inputStreamReader != null) {
					inputStreamReader.close();
				}

				if (inputStream != null) {
					inputStream.close();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return false;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		OrderStatusNotifier notifier = new OrderStatusNotifier();
		String orderId = "201503301956491233487414";
		OrderStatus status = OrderStatus.canceled;
		String note = "服务人员已满";
		if (notifier.notifyDaoWay(orderId, status, note)) {
			System.out.println("notify succeed");
		} else {
			System.out.println("notify failed");
		}

	}

}
